@@ -60,12 +60,12 @@ logger.addHandler(f_handler)
 
 class DataCulpaConnectionError(Exception):
     def __init__(self, url, message):
-        logger.error("Connection error for url %s: __%s__" % (url, message))
+        #logger.error("Connection error for url %s: __%s__" % (url, message))
         super().__init__("Connection error for URL %s: __%s__" % (url, message))
 
 class DataCulpaServerResponseParseError(Exception):
     def __init__(self, url, payload):
-        logger.error("Error parsing result from url %s: __%s__" % (url, payload))
+        #logger.error("Error parsing result from url %s: __%s__" % (url, payload))
         super().__init__("Bad response from URL %s: __%s__" % (url, payload))
 
 class DataCulpaBadServerCodeError(Exception):
@@ -140,13 +140,11 @@ class DataCulpaValidator:
 
     @classmethod
     def GetWatchpointVariations(cls, protocol, host, port, watchpoint_name):
-        try:
-            pName = base64.urlsafe_b64encode(watchpoint_name.encode('utf-8')).decode('utf-8')
-            url = "%s://%s:%s/%s" % (protocol, host, port, "data/metadata/watchpoint-variations/%s" % pName)
 
-            r = DataCulpaValidator.GET(url)
-        except:
-            raise DataCulpaConnectionError
+        pName = base64.urlsafe_b64encode(watchpoint_name.encode('utf-8')).decode('utf-8')
+        url = "%s://%s:%s/%s" % (protocol, host, port, "data/metadata/watchpoint-variations/%s" % pName)
+
+        r = DataCulpaValidator.GET(url)
         
         try:
             jr = DataCulpaValidator._parseJson(url, r.content)
@@ -174,6 +172,8 @@ class DataCulpaValidator:
         except requests.RequestException as e:
             raise DataCulpaConnectionError(url, "request error: %s" % e)
         except BaseException as e:
+            if isinstance(e, DataCulpaBadServerCodeError):
+                raise e # Don't wrap the exception we just made above.
             raise DataCulpaConnectionError(url, "unexpected error: %s" % e)
         return None
 
@@ -187,7 +187,16 @@ class DataCulpaValidator:
                               timeout=timeout,
                               headers=headers)
             if r.status_code != 200:
-                raise DataCulpaBadServerCodeError(r.status_code, "url was %s" % url)
+                new_text = r.text
+                # Chop off the annoying HTML nonsense if it's there..
+                if new_text.startswith("<!DOCTYPE"):
+                    p_pos = new_text.find("<p>")
+                    if p_pos >= 0:
+                        new_text = new_text[p_pos + 3:]
+                        p_pos = new_text.find("</p>")
+                        if p_pos > 0:
+                            new_text = new_text[:p_pos]
+                raise DataCulpaBadServerCodeError(r.status_code, "url was %s; text = %s" % (r.url, new_text))
             return r
         except requests.exceptions.Timeout:
             raise DataCulpaConnectionError(url, "timed out")
@@ -196,6 +205,8 @@ class DataCulpaValidator:
         except requests.RequestException as e:
             raise DataCulpaConnectionError(url, "request error: %s" % e)
         except BaseException as e:
+            if isinstance(e, DataCulpaBadServerCodeError):
+                raise e # Don't wrap the exception we just made above.
             raise DataCulpaConnectionError(url, "unexpected error: %s" % e)
         return None
 
@@ -429,6 +440,7 @@ class DataCulpaValidator:
                 dt = int(dt.total_seconds())
                 return dt
 
+        assert False, "Unexpected type for timeshift - should an int/float of seconds or a datetime, but got a %s" % type(ts)
         return 0
 
 
@@ -470,6 +482,20 @@ class DataCulpaValidator:
 
         return # (None, 0, 0)
 
+    def queue_fastqueue(self, field_name, file_path):
+        """Zero-copy: the file-path is on a shared filesystem with Validator"""
+
+        assert len(self._queue_buffer) == 0, "fastqueue cannot be mixed with incremental queueing."
+
+        queue_id = self._queue_id
+        assert queue_id is not None
+
+        url = self._get_base_url("queue/fastqueue/%s" % queue_id)
+        params = { "field_name": file_path, "field_name": field_name }
+        r = self.POST(url, params)
+        jr = self._parseJson(url, r.content)
+        return (queue_id, jr)
+
     def queue_commit(self):
         """Returns the queue_id if successful -- so that we can query for validation status on that queue_id.
            By the time this returns, the queue has been closed--so the only utility is to query for validation status
